local LubStub = LibStub
local DeathClock = LibStub('AceAddon-3.0'):NewAddon('DeathClock', 'AceEvent-3.0', 'AceConsole-3.0')
_G.DeathClock = DeathClock
local SML = LibStub("LibSharedMedia-3.0")

local registerOnShow = function(frame) 
	frame.health0, frame.time0, frame.mhealth, frame.mtime = nil
	frame:RegisterEvent("UNIT_HEALTH") 
end

local unregisterOnHide = function(frame) 
	frame:UnregisterEvent("UNIT_HEALTH")
end

local startDrag = function(frame) 
	if IsAltKeyDown() then 
		frame:StartMoving() 
	end 
end

local stopDrag = function(frame) 
	frame:StopMovingOrSizing() 
end

local timetodieUpdate = function(frame, e, unit) 

	if e == frame.resetEvent then
		frame.health0, frame.time0, frame.mhealth, frame.mtime = nil
		frame.text:SetText("...")
		return
	end

	local health = UnitHealth(frame.target)

	if health == UnitHealthMax(frame.target) then
		frame.health0, frame.time0, frame.mhealth, frame.mtime = nil
		frame.text:SetText("...")
		return
	end

	local time = GetTime()

	if not frame.health0 then
		frame.health0, frame.time0 = health, time
		frame.mhealth, frame.mtime = health, time
		return
	end

	frame.mhealth = (frame.mhealth + health) * .5
	frame.mtime = (frame.mtime + time) * .5

	if frame.mhealth > frame.health0 then
		frame.text:SetText("...")
		frame.health0, frame.time0, frame.mhealth, frame.mtime = nil
	else
		time = ceil(health * (frame.time0 - frame.mtime) / (frame.mhealth - frame.health0) )
		if time < 60 then
			frame.text:SetText(format('%ds',time))
		elseif time < 3600 then
			frame.text:SetText(format('%dm%02ds', 1/60 * time, time % 60))
		else
			frame.text:SetText(format('%dh%02dm', 1/3600 * time, 1/60 * (time % 3600)))
		end
	end
end

function setupFrame(frameName,parent,textName,target,resetEvent)
	local frame = CreateFrame("Frame", frameName, parent)
	frame:SetPoint("LEFT", parent, "LEFT", -48, -12)
	frame:SetWidth(50)
	frame:SetHeight(20)
	frame.text = frame:CreateFontString(textName, "OVERLAY")
	frame.text:SetAllPoints(frame)
	frame.text:SetFontObject(TextStatusBarText)
	frame.text:SetJustifyH("RIGHT")
	if resetEvent then
		frame:RegisterEvent(resetEvent)
		frame.resetEvent = resetEvent
	else
		frame.resetEvent = "NoResetEvent"
	end
	frame:SetScript("OnShow", registerOnShow)
	frame:SetScript("OnHide", unregisterOnHide)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetClampedToScreen(true)
	frame:SetMovable(true)
	frame:SetScript("OnDragStart", startDrag)
	frame:SetScript("OnDragStop", stopDrag)
	frame:SetScript("OnEvent", timetodieUpdate)
	frame.target = target
	return frame
end

local t_dc =  setupFrame("TargetDeathClock", TargetFrameHealthBar,      "TargetDeathClockText", "target", "PLAYER_TARGET_CHANGED")
local f_dc =  setupFrame("FocusDeathClock",  FocusFrameHealthBar,       "FocusDeathClockText",  "focus",  "PLAYER_FOCUS_CHANGED")
local b1_dc = setupFrame("Boss1DeathClock",  Boss1TargetFrameHealthBar, "Boss1DeathClockText",  "boss1",  nil)
local b2_dc = setupFrame("Boss2DeathClock",  Boss2TargetFrameHealthBar, "Boss2DeathClockText",  "boss2",  nil)
local b3_dc = setupFrame("Boss3DeathClock",  Boss3TargetFrameHealthBar, "Boss3DeathClockText",  "boss3",  nil)
local b4_dc = setupFrame("Boss4DeathClock",  Boss4TargetFrameHealthBar, "Boss4DeathClockText",  "boss4",  nil)